@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# Copyright 2005-2007 Messiah College. All rights reserved.
+# Copyright 2005-2006 Messiah College. All rights reserved.
 # Jason Long <jlong@messiah.edu>
 
 # Copyright (c) 2004 Anthony D. Urso. All rights reserved.
@@ -32,32 +32,48 @@ sub init
 {
 	my $self = shift;
 
-	croak "no signature" unless $self->{Signature};
+	$self->{debug_buf} = "";
 
-	$self->{mode} = $self->{Signature}->signature ? "verify" : "sign";
-	$self->{draft_version} ||=
-		($self->{mode} eq "sign" ? "01" :
-		 $self->{Signature}->body_hash ? "01" : "00");
+	if ($self->{Signature})
+	{
+		$self->{mode} = $self->{Signature}->signature ? "verify" : "sign";
+		$self->{draft_version} ||=
+			($self->{mode} eq "sign" ? "01" :
+			 $self->{Signature}->body_hash ? "01" : "00");
 
-	# allows subclasses to set the header_digest and body_digest
-	# properties
-	$self->init_digests;
+		# allows subclasses to set the header_digest and body_digest
+		# properties
+		$self->init_digests;
 
-	my ($header_method, $body_method)
-		= $self->{Signature}->canonicalization;
+		my ($header_method, $body_method)
+			= $self->{Signature}->canonicalization;
 
-	my $header_class = $self->get_canonicalization_class($header_method);
-	my $body_class = $self->get_canonicalization_class($body_method);
-	$self->{canon} = $header_class->new(
-			output_digest => $self->{header_digest},
-			draft_version => $self->{draft_version},
-			Signature => $self->{Signature},
-			Debug_Canonicalization => $self->{Debug_Canonicalization});
-	$self->{body_canon} = $body_class->new(
-			output_digest => $self->{body_digest},
-			draft_version => $self->{draft_version},
-			Signature => $self->{Signature},
-			Debug_Canonicalization => $self->{Debug_Canonicalization});
+		my ($header_buffer, $body_buffer);
+		if ($self->{Debug_Canonicalization})
+		{
+			$self->{debug_buf} = "";
+			$header_buffer = \$self->{debug_buf};
+			$self->{debug_body_buf} = "";
+			$body_buffer = \$self->{debug_body_buf};
+		}
+
+		my $header_class = $self->get_canonicalization_class($header_method);
+		my $body_class = $self->get_canonicalization_class($body_method);
+		$self->{canon} = $header_class->new(
+				buffer => $header_buffer,
+				output_digest => $self->{header_digest},
+				draft_version => $self->{draft_version},
+				Signature => $self->{Signature});
+		$self->{body_canon} = $body_class->new(
+				buffer => $body_buffer,
+				output_digest => $self->{body_digest},
+				draft_version => $self->{draft_version},
+				Signature => $self->{Signature});
+	}
+	else
+	{
+		die "no signature";
+	}
 }
 
 # override this method, please...
@@ -67,7 +83,7 @@ sub init_digests
 	die "not implemented";
 }
 
-# private method - DKIM-specific
+# private method
 sub get_canonicalization_class
 {
 	my $self = shift;
@@ -81,6 +97,35 @@ sub get_canonicalization_class
 	return $class;
 }
 
+#sub TIEHANDLE
+#{
+#	my $class = shift;
+#	return $class->new(@_);
+#}
+#
+## override this method, please...
+#sub PRINT
+#{
+#	my $self = shift;
+#
+#	if (my $debug = $self->{Debug_Canonicalization})
+#	{
+#		if (ref($debug) && ref($debug) eq "SCALAR")
+#		{
+#			$$debug .= join("", @_);
+#		}
+#		else
+#		{
+#			$self->{debug_buf} .= join("", @_);
+#		}
+#	}
+#}
+#
+#sub CLOSE
+#{
+#}
+
+
 =head1 NAME
 
 Mail::DKIM::Algorithm::Base - base class for DKIM "algorithms"
@@ -119,15 +164,12 @@ of the DKIM algorithm implementation classes, such as rsa_sha1:
   $algorithm->add_body("This is the body.\015\012");
   $algorithm->add_body("Another line of the body.\015\012");
 
-The body should be fed one "line" at a time.
-
 =cut
 
 sub add_body
 {
 	my $self = shift;
-	my $canon = $self->{body_canon} || $self->{canon};
-	$canon->add_body(@_);
+	$self->{body_canon}->add_body(@_);
 }
 
 =head2 add_header() - feeds a header field into the algorithm/canonicalization
@@ -159,8 +201,7 @@ from the algorithm.
 sub finish_body
 {
 	my $self = shift;
-	my $body_canon = $self->{body_canon} || $self->{canon};
-	$body_canon->finish_body;
+	$self->{body_canon}->finish_body;
 	$self->finish_message;
 }
 
@@ -186,10 +227,6 @@ sub check_body_hash
 {
 	my $self = shift;
 
-	# The body_hash value is set in finish_message(), if we're operating
-	# from a version of the DKIM spec that uses the bh= tag. Otherwise,
-	# the signature shouldn't have a bh= tag to check.
-
 	if ($self->{body_hash})
 	{
 		my $body_hash = $self->{body_hash};
@@ -210,10 +247,7 @@ sub check_body_hash
 sub finish_message
 {
 	my $self = shift;
-
-	# DKIM requires the signature itself to be committed into the digest.
-	# But first, we need to set the bh= tag on the signature, then
-	# "prettify" it.
+#	$self->{canon}->finish_message;
 
 	if ($self->{draft_version} eq "01")
 	{
@@ -223,11 +257,11 @@ sub finish_message
 			$self->{Signature}->body_hash(
 					encode_base64($self->{body_hash}, ""));
 		}
-	}
-
-	if ($self->{mode} eq "sign")
-	{
-		$self->{Signature}->prettify;
+	#	else
+	#	{
+	#		print STDERR "verify: body hash is "
+	#			. encode_base64($self->{body_hash}, "") . "\n";
+	#	}
 	}
 
 	my $sig_line = $self->{Signature}->as_string_without_data;
@@ -238,7 +272,28 @@ sub finish_message
 		$canonicalized = "\015\012" . $canonicalized;
 	}
 
-	$self->{canon}->output($canonicalized);
+	$self->{header_digest}->add($canonicalized);
+
+	if (my $debug = $self->{Debug_Canonicalization})
+	{
+		$self->{debug_buf} .= $canonicalized;
+		unless (ref $debug)
+		{
+			my $filename = $debug;
+			open my $fh, ">", $filename
+				or die "Error: cannot write to $filename: $!\n";
+			print $fh "-----BEGIN CANONICALIZED HEADERS-----\015\012";
+			print $fh $self->{debug_buf};
+			print $fh "-----END CANONICALIZED HEADERS-----\015\012";
+			print $fh "-----BEGIN CANONICALIZED BODY-----\015\012";
+			print $fh $self->{debug_body_buf};
+			print $fh "-----END CANONICALIZED BODY-----\015\012";
+			close $fh;
+			print STDERR "Debug: wrote canonicalized headers and body to $filename\n";
+		#	print STDERR "Body count is " . $self->{body_canon}->body_count . "\n";
+		#	print STDERR "Truncated " . $self->{body_canon}->{body_truncated} . "\n";
+		}
+	}
 }
 
 =head2 sign() - generates a signature using a private key
@@ -253,32 +308,12 @@ sub sign
 	die "Not implemented";
 }
 
-=head2 signature() - get/set the signature worked on by this algorithm
+=head2 verify() - verifies a signature using the public key
 
-  my $old_signature = $algorithm->signature;
-  $algorithm->signature($new_signature);
-
-=cut
-
-sub signature
-{
-	my $self = shift;
-	@_ and
-		$self->{Signature} = shift;
-	return $self->{Signature};
-}
-
-=head2 verify() - verifies a signature
-
-  $result = $algorithm->verify();
-
-Must be called after finish_body().
+  $result = $algorithm->verify($base64, $public_key);
 
 The result is a true/false value: true indicates the signature data
-is valid, false indicates it is invalid.
-
-For an invalid signature, details may be obtained from
-$algorithm->{verification_details} or $@.
+($base64) is valid, false indicates it is invalid.
 
 =cut
 
@@ -289,23 +324,3 @@ sub verify
 }
 
 1;
-
-__END__
-
-=head1 SEE ALSO
-
-L<Mail::DKIM>
-
-=head1 AUTHOR
-
-Jason Long, E<lt>jlong@messiah.eduE<gt>
-
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2005-2007 by Messiah College
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.6 or,
-at your option, any later version of Perl 5 you may have available.
-
-=cut
